@@ -1,6 +1,7 @@
 """skimpy provides summary statistics about variables in pandas data frames."""
 import re
 from collections import defaultdict
+from dataclasses import MISSING
 from itertools import chain
 from typing import Any
 from typing import Dict
@@ -13,6 +14,7 @@ import pandas as pd
 import rich
 from numpy.random import Generator
 from numpy.random import PCG64
+from rich import print
 from rich.columns import Columns
 from rich.console import Console
 from rich.panel import Panel
@@ -52,10 +54,12 @@ UNICODE_HIST = {
 DATE_COL_FIRST = "first"
 DATE_COL_LAST = "last"
 NUM_COL_MEAN = "mean"
+COMPLETE_COL = "NA %"
+MISSING_COL = "NA"
 
 
 @typechecked
-def infer_datatypes(df: pd.DataFrame) -> pd.DataFrame:
+def _infer_datatypes(df: pd.DataFrame) -> pd.DataFrame:
     """Infers the, and applies new, datatypes of dataframe columns.
 
     :param df: input dataframe of ambiguous col type
@@ -89,7 +93,7 @@ def infer_datatypes(df: pd.DataFrame) -> pd.DataFrame:
 
 
 @typechecked
-def round_dataframe(df: pd.DataFrame) -> pd.DataFrame:
+def _round_dataframe(df: pd.DataFrame) -> pd.DataFrame:
     """Rounds dataframe to 2 s.f.
 
     Args:
@@ -104,7 +108,7 @@ def round_dataframe(df: pd.DataFrame) -> pd.DataFrame:
 
 
 @typechecked
-def map_row_positions_to_text_style(types_to_property: dict, df: pd.DataFrame) -> dict:
+def _map_row_positions_to_text_style(types_to_property: dict, df: pd.DataFrame) -> dict:
     """Maps positions in summary dataframe (eg row) to a Rich text property.
 
     :param types_to_property: Datatype, datetime, mapping to Rich text property
@@ -141,7 +145,7 @@ def map_row_positions_to_text_style(types_to_property: dict, df: pd.DataFrame) -
 
 
 @typechecked
-def simplify_datetimes_in_array(rows: np.ndarray) -> np.ndarray:
+def _simplify_datetimes_in_array(rows: np.ndarray) -> np.ndarray:
     """Simplifies 2001/01/01 00:00:00 to 2001/01/01.
 
     :param rows: contain summary info, including datetimes
@@ -167,7 +171,7 @@ def simplify_datetimes_in_array(rows: np.ndarray) -> np.ndarray:
 
 
 @typechecked
-def dataframe_to_rich_table(
+def _dataframe_to_rich_table(
     table_name: str,
     df: pd.DataFrame,
     number: str = "cyan",
@@ -201,7 +205,7 @@ def dataframe_to_rich_table(
         rich.table.Table: instance of Table from the rich package
     """
     str_limit: int = 20  # For longer strings, limit chars shown.
-    df = df.reset_index().rename(columns={"index": ""})
+    df = df.reset_index().rename(columns={"index": "column_name"})
     table = Table(show_footer=False, expand=True, title=table_name, show_header=True)
     # generate dict of types to colours
     datatype_colours = {
@@ -221,12 +225,12 @@ def dataframe_to_rich_table(
         "bool": "left",
         "object": "left",
     }
-    pos_to_colour = map_row_positions_to_text_style(datatype_colours, df)
-    pos_to_justification = map_row_positions_to_text_style(datatype_justify, df)
+    pos_to_colour = _map_row_positions_to_text_style(datatype_colours, df)
+    pos_to_justification = _map_row_positions_to_text_style(datatype_justify, df)
     rows = df.values
     # find any datetimes
     if (DATE_COL_FIRST or DATE_COL_LAST) in df.columns:
-        rows = simplify_datetimes_in_array(rows)
+        rows = _simplify_datetimes_in_array(rows)
     rows = [
         [
             str(s).rstrip("0").rstrip(".") if ("." and type(s) == float) else s
@@ -249,7 +253,7 @@ def dataframe_to_rich_table(
     return table
 
 
-def find_nearest(array, value):
+def _find_nearest(array, value):
     """Find the nearest numerical match to value in an array.
 
     Args:
@@ -265,7 +269,7 @@ def find_nearest(array, value):
 
 
 @typechecked
-def create_unicode_hist(series: pd.Series) -> pd.Series:
+def _create_unicode_hist(series: pd.Series) -> pd.Series:
     """Return a histogram rendered in block unicode.
 
     Given a pandas series of numerical values, returns a series with one
@@ -286,13 +290,13 @@ def create_unicode_hist(series: pd.Series) -> pd.Series:
     # now do value counts
     key_vector = np.array(list(UNICODE_HIST.keys()), dtype="float")
     ucode_to_print = "".join(
-        [UNICODE_HIST[find_nearest(key_vector, val)] for val in hist]
+        [UNICODE_HIST[_find_nearest(key_vector, val)] for val in hist]
     )
     return pd.Series(index=[series.name], data=ucode_to_print, dtype="string")
 
 
 @typechecked
-def numeric_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
+def _numeric_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
     """Summarise dataframe columns that have numeric type.
 
     Args:
@@ -304,8 +308,8 @@ def numeric_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
     """
     count_nans_vec = xf.isna().sum()
     data_dict = {
-        "missing": count_nans_vec,
-        "complete rate": 1 - count_nans_vec / xf.shape[0],
+        MISSING_COL: count_nans_vec,
+        COMPLETE_COL: 100 * count_nans_vec / xf.shape[0],
         NUM_COL_MEAN: xf.mean(),
         "sd": xf.std(),
     }
@@ -320,7 +324,7 @@ def numeric_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
     # Create histogram using unicode block elements
     # https://en.wikipedia.org/wiki/Block_Elements
     hist_series = pd.concat(
-        [create_unicode_hist(xf[col].dropna()) for col in xf.columns], axis=0
+        [_create_unicode_hist(xf[col].dropna()) for col in xf.columns], axis=0
     )
     data_dict.update({"hist": hist_series})
     summary_df = pd.DataFrame(data_dict)
@@ -328,7 +332,7 @@ def numeric_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
 
 
 @typechecked
-def category_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
+def _category_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
     """Summarise dataframe columns that have category type.
 
     Args:
@@ -340,8 +344,8 @@ def category_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
     """
     count_nans_vec = xf.isna().sum()
     data_dict = {
-        "missing": count_nans_vec,
-        "complete rate": 1 - count_nans_vec / xf.shape[0],
+        MISSING_COL: count_nans_vec,
+        COMPLETE_COL: 100 * count_nans_vec / xf.shape[0],
         "ordered": pd.Series(
             dict(zip(xf.columns, [xf[col].cat.ordered for col in xf.columns]))
         ),
@@ -354,7 +358,7 @@ def category_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
 
 
 @typechecked
-def bool_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
+def _bool_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
     """Summarise dataframe columns that have boolean type.
 
     Args:
@@ -369,7 +373,7 @@ def bool_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
         "true rate": xf.sum() / xf.shape[0],
     }
     hist_series = pd.concat(
-        [create_unicode_hist(xf[col].dropna()) for col in xf.columns], axis=0
+        [_create_unicode_hist(xf[col].dropna()) for col in xf.columns], axis=0
     )
     data_dict.update({"hist": hist_series})
     summary_df = pd.DataFrame(data_dict)
@@ -377,7 +381,7 @@ def bool_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
 
 
 @typechecked
-def string_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
+def _string_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
     """Summarise dataframe columns that have string type. (NB not object type).
 
     Args:
@@ -389,8 +393,8 @@ def string_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
     """
     count_nans_vec = xf.isna().sum()
     data_dict = {
-        "missing": count_nans_vec,
-        "complete rate": 1 - count_nans_vec / xf.shape[0],
+        MISSING_COL: count_nans_vec,
+        COMPLETE_COL: 100 * count_nans_vec / xf.shape[0],
         "words per row": pd.Series(
             dict(
                 zip(
@@ -419,7 +423,7 @@ def string_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
 
 
 @typechecked
-def datetime_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
+def _datetime_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
     """Summarise dataframe columns that have datetime type.
 
     Args:
@@ -431,8 +435,8 @@ def datetime_variable_summary_table(xf: pd.DataFrame) -> pd.DataFrame:
     """
     count_nans_vec = xf.isna().sum()
     data_dict = {
-        "missing": count_nans_vec,
-        "complete rate": 1 - count_nans_vec / xf.shape[0],
+        MISSING_COL: count_nans_vec,
+        COMPLETE_COL: 100 * count_nans_vec / xf.shape[0],
         DATE_COL_FIRST: pd.Series(
             dict(
                 zip(
@@ -470,12 +474,21 @@ def skim(
     functions based on the types of columns in the dataframe. You may get
     better results from ensuring that you set the datatypes in your dataframe
     you want before running skim.
-    The colour_kwargs (str) are defined in dataframe_to_rich_table.
+    The colour_kwargs (str) are defined in _dataframe_to_rich_table.
 
     Args:
         df (pd.DataFrame): Dataframe to skim
         header_style (str): A style to use for headers. See Rich API Styles.
         colour_kwargs (dict[str]): colour keyword arguments for rich table
+
+    Examples
+    --------
+    Skim a dataframe
+    >>> df = pd.DataFrame({'col1': ['Philip', 'Turanga', 'bob'], \
+        'col2': [50, 100, 70], \
+        'col3': [False, True, True]})
+    >>> df["col1"] = df["col1"].astype("string")
+    >>> skim(df)
     """
     if hasattr(df, "name") and "name" not in df.columns:
         name = df.name
@@ -483,7 +496,7 @@ def skim(
         name = "dataframe"
 
     # Perform inference of datatypes
-    # df = infer_datatypes(df)
+    df = _infer_datatypes(df)
 
     # Data summary
     tab_1_data = {"Number of rows": df.shape[0], "Number of columns": df.shape[1]}
@@ -516,11 +529,11 @@ def skim(
             cat_sum_table.add_row(cat)
     # Summaries of cols of specific types
     types_funcs_dict = {
-        "number": numeric_variable_summary_table,
-        "category": category_variable_summary_table,
-        "datetime": datetime_variable_summary_table,
-        "string": string_variable_summary_table,
-        "bool": bool_variable_summary_table,
+        "number": _numeric_variable_summary_table,
+        "category": _category_variable_summary_table,
+        "datetime": _datetime_variable_summary_table,
+        "string": _string_variable_summary_table,
+        "bool": _bool_variable_summary_table,
     }
     list_of_tabs = []
     for col_type, summary_func in types_funcs_dict.items():
@@ -528,8 +541,8 @@ def skim(
         if not xf.empty:
             sum_df = summary_func(xf)
             list_of_tabs.append(
-                dataframe_to_rich_table(
-                    col_type, round_dataframe(sum_df)  # , **colour_kwargs
+                _dataframe_to_rich_table(
+                    col_type, _round_dataframe(sum_df)  # , **colour_kwargs
                 )
             )
     # Put all of the info together
@@ -555,8 +568,7 @@ def clean_columns(
 ) -> pd.DataFrame:
     """Function to clean column names, originally from the dataprep python package.
 
-    Parameters
-    ----------
+    Args:
     df
         Dataframe from which column names are to be cleaned.
     case
@@ -578,22 +590,22 @@ def clean_columns(
     remove_accents
         If True, strip accents from the column names.
         (default: True)
+
     Examples
     --------
     Clean column names by converting the names to camel case style, removing accents,
     and correcting a mispelling.
-    >>> df = pd.DataFrame({'FirstNom': ['Philip', 'Turanga'], 'lastName': ['Fry', 'Leela'], \
-'Téléphone': ['555-234-5678', '(604) 111-2335']})
-    >>> clean_headers(df, case='camel', replace={'Nom': 'Name'})
-    Column Headers Cleaning Report:
-        2 values cleaned (66.67%)
+    >>> df = pd.DataFrame({'FirstNom': ['Philip', 'Turanga'], \
+        'lastName': ['Fry', 'Leela'], \
+        'Téléphone': ['555-234-5678', '(604) 111-2335']})
+    >>> clean_columns(df, case='camel', replace={'Nom': 'Name'})
       firstName lastName       telephone
     0    Philip      Fry    555-234-5678
     1   Turanga    Leela  (604) 111-2335
     """
     if case not in CASE_STYLES:
         raise ValueError(
-            f"case {case} is invalid, it needs to be one of {', '.join(c for c in CASE_STYLES)}"
+            f"case {case} is invalid, options are: {', '.join(c for c in CASE_STYLES)}"
         )
 
     # Store original column names for creating cleaning report
@@ -612,8 +624,7 @@ def clean_columns(
     cleaned = [
         1 if new_columns[i] != orig_columns[i] else 0 for i in range(len(orig_columns))
     ]
-    stats = {"cleaned": sum(cleaned)}
-
+    print(f"{sum(cleaned)} column names have been cleaned")
     return df
 
 
@@ -621,12 +632,12 @@ def clean_columns(
 def _convert_case(name: Any, case: str) -> Any:
     """Convert case style of a column name.
 
-    Parameters
-    ----------
-    name
-        Column name.
-    case
-        The desired case style of the column name.
+    Args:
+        name (Any): Column name.
+        case (str): Preferred case type, eg snake or camel.
+
+    Returns:
+        Any: name with case converted.
     """
     if name in NULL_VALUES:
         name = "header"
@@ -696,11 +707,11 @@ def _replace_values(name: Any, mapping: Dict[str, str]) -> Any:
         # If the old value or the new value is not alphanumeric, add underscores to the
         # beginning and end so the new value will be parsed correctly for _convert_case()
         new_val = (
-            fr"{new_value}"
+            rf"{new_value}"
             if old_value.isalnum() and new_value.isalnum()
-            else fr"_{new_value}_"
+            else rf"_{new_value}_"
         )
-        name = re.sub(fr"{old_value}", new_val, name, flags=re.IGNORECASE)
+        name = re.sub(rf"{old_value}", new_val, name, flags=re.IGNORECASE)
 
     return name
 
@@ -747,6 +758,11 @@ def generate_test_data() -> pd.DataFrame:
 
     Returns:
         pd.DataFrame: dataframe with columns spanning several data types.
+
+    Examples
+    --------
+    Generate test data to demonstrate how skimpy works.
+    >>> df = generate_test_data()
     """
     seed = 34729
     rng = Generator(PCG64(seed))
